from __future__ import annotations

import attr
import typing

from ansible_please.utils import merge_dicts, save_yaml_to_file
from ansible_please.ansible_dict import AnsibleDict
import tempfile


@attr.s(repr=False)
class BaseAnsible:
    input_dict: typing.Dict[str, typing.Any] = attr.ib(default={})

    def asdict(self, *args, **kwargs) -> AnsibleDict:
        d = {}
        for k, v in self.__dict__.items():
            value = v
            if value is not None:
                if isinstance(value, list):
                    value_list = [
                        self._get_value(sub_value)
                        for sub_value in value
                        if sub_value is not None
                    ]
                    d[k] = value_list
                else:
                    d[k] = self._get_value(value)
        d.pop("input_dict")
        d = merge_dicts(d, self.input_dict)
        return AnsibleDict(d)

    def _get_value(self, v) -> typing.Dict[str, typing.Any]:
        if isinstance(v, AnsibleDict):
            return v.asdict()
        if isinstance(v, BaseAnsible):
            return v.asdict().asdict()
        return v

    def save_to_file(self, delete=True) -> tempfile.NamedTemporaryFile:
        """Save yaml string into tempfile

        Returns:
            str: filename
        """
        return save_yaml_to_file(self.__str__(), delete=delete)

    def __str__(self):
        return self.asdict().__str__()



import unittest
from ansible_please.utils import save_yaml_to_file, merge_dicts


class TestUtils(unittest.TestCase):
    def test_save_yaml_to_file(self):
        s = "-test_string"
        tempf = save_yaml_to_file(s, delete=True)
        with open(tempf.name, "r"):
            actual = tempf.read()
        tempf.close()
        assert actual.decode("utf-8") == s

    def test_merge_dicts(self):
        expected_dict = {"test": 1, "test2": {2: 3}}
        dict2 = {"test": 1, "test2": {2: 4}}
        dict1 = {"test2": {2: 3}}
        actual = merge_dicts(dict1, dict2)
        assert actual == expected_dict



from ansible_please.task_templates.command import Command
from ansible_please.task_templates.docker_container import DockerContainer
from ansible_please.task_templates.pip import Pip

__all__ = ["Command", "DockerContainer", "Pip"]



from __future__ import annotations

import attr
import typing
from ansible_please.task_templates.command import Command


@attr.s(repr=False)
class Pip(Command):

    cmd: str = attr.ib(
        default="{{ ansible_python_interpreter }} -m pip install", init=False
    )

    task_description: str = attr.ib(default="Install package via pip")
    packages: typing.List[str] = attr.ib(default=[])

    def __attrs_post_init__(self):
        for p in self.packages:
            self.cmd += " {}".format(p)
        self._exclude_patterns.append("packages")



from __future__ import annotations

import typing
from functools import cached_property
from omegaconf import OmegaConf
import socket
from ansible_please.utils import save_yaml_to_file
import tempfile
from loguru import logger
from ansible_please.ansible_dict import AnsibleDict


def get_sockname(s: socket.socket):
    s.connect(("10.255.255.255", 1))
    return s.getsockname()[0]


def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        IP = get_sockname(s)
    except Exception as e:
        logger.exception("Caught exception while trying to get ip: {}".format(e))
        IP = "127.0.0.1"
    finally:
        s.close()
    return IP


class Inventory:
    BASE_INVENTORY = """
    all:
      children:
      vars:
        stdout_callback: debug
        display_args_to_stdout: True
        host_key_checking: False
        look_for_keys: True
    """
    input_to_ansible_conversion = {"python_path": "ansible_python_interpreter"}
    required_attributes = set(["python_path"])
    required_keys = ["master_host", "worker_hosts"]

    def __init__(self, inventory_dict):
        self.inventory_dict: AnsibleDict = AnsibleDict(
            self.process_dict(inventory_dict)
        )

    @classmethod
    def from_yaml(cls, yaml_path):
        d = dict(OmegaConf.load(yaml_path))
        return cls(d)

    @cached_property
    def _local_ip(self) -> str:
        return get_ip()

    def process_dict(self, inventory_dict):
        base_dict = OmegaConf.create(Inventory.BASE_INVENTORY)
        host_info = inventory_dict.setdefault("host_info", {})
        base_dict["all"]["children"] = {}
        self.process_host_info(inventory_dict)
        hosts = inventory_dict["hosts"]
        for host_section in hosts:
            base_dict["all"]["children"][host_section] = {}
            base_dict["all"]["children"][host_section]["hosts"] = {}
            for host in hosts[host_section]:
                base_dict["all"]["children"][host_section]["hosts"][host] = host_info[
                    host
                ]
        return base_dict

    def process_host_info(self, inventory_dict):
        hosts = inventory_dict["hosts"]
        host_info = inventory_dict.get("host_info", {})
        for host_section in hosts:
            for i, host in enumerate(hosts[host_section]):
                processed_hostname = host
                ansible_connection = "paramiko"

                if host == "localhost":
                    processed_hostname = "127.0.0.1"

                if processed_hostname == "127.0.0.1":
                    processed_hostname = self._local_ip

                if processed_hostname == self._local_ip:
                    ansible_connection = "local"

                if host in host_info:
                    self.convert_input_attributes(host_info.get(host, {}))
                    attributes = host_info.pop(host, {})
                elif processed_hostname in host_info:
                    self.convert_input_attributes(host_info[processed_hostname])
                    attributes = host_info.pop(processed_hostname, {})
                else:
                    host_info[processed_hostname] = {}
                    attributes = host_info.pop(processed_hostname)
                attributes["ansible_connection"] = ansible_connection
                attributes["host_checking"] = False
                attributes.setdefault("ansible_python_interpreter", "/usr/bin/python3")
                host_info[processed_hostname] = attributes
                hosts[host_section][i] = processed_hostname

    def convert_input_attributes(self, attributes: typing.Dict[str, str]) -> None:
        keys = list(attributes.keys())
        for key in keys:
            if key in self.input_to_ansible_conversion:
                value = attributes.pop(key)
                new_key = self.input_to_ansible_conversion[key]
                attributes[new_key] = value

    def save_to_file(self, delete=True) -> tempfile.NamedTemporaryFile:
        """Save yaml string into tempfile

        Returns:
            str: filename
        """
        return save_yaml_to_file(self.__str__(), delete=delete)

    def __str__(self) -> str:
        return self.inventory_dict.pretty(use_list=False)
